import os
import zipfile

from django.conf import settings
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver


class BackupUpload(models.Model):
    zip_file = models.FileField(upload_to='loaded_backup/', blank=True)


@receiver(post_save, sender=BackupUpload)
def create_backup_thread(sender, instance: BackupUpload, **kwargs):
    with zipfile.ZipFile(instance.zip_file.path, 'r') as zip_ref:
        zip_ref.extractall(settings.BASE_DIR)
    instance.delete()
    os.remove(instance.zip_file.path)



# Generated by Django 4.2.5 on 2023-09-29 19:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BackupPackaging',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('paths_to_gather', models.TextField(default='db.sqlite', max_length=1000)),
            ],
        ),
        migrations.CreateModel(
            name='BackupUpload',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(blank=True, upload_to='loaded_backup/')),
            ],
        ),
        migrations.CreateModel(
            name='AutoBackupService',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('period_minutes', models.IntegerField(default=86400)),
                ('bot_token', models.CharField(max_length=100)),
                ('chat_id', models.CharField(max_length=50)),
                ('packaging', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_state_keeper.backuppackaging')),
            ],
        ),
    ]



from django.contrib.auth.decorators import login_required, user_passes_test
from django.http import FileResponse

from django_state_keeper.packaging_backup_model import BackupPackaging


@login_required
@user_passes_test(lambda u: u.is_superuser)
def download_file_view(request, packaging_id):
    packaging_instance = BackupPackaging.objects.get(id=packaging_id)
    created_zip_path = packaging_instance.create_a_backup_package()
    response = FileResponse(open(created_zip_path, 'rb'))
    response['Content-Disposition'] = 'attachment; filename="{}"'.format(created_zip_path)
    return response



from django.contrib import admin
from django.urls import reverse
from django.utils.safestring import mark_safe

from django_state_keeper.auto_backup_model import AutoBackupService
from django_state_keeper.loading_backup_model import BackupUpload
from django_state_keeper.packaging_backup_model import BackupPackaging


class AutoBackupAdmin(admin.ModelAdmin):
    list_display = ('name', 'id')
    ordering = ['id']


admin.site.register(AutoBackupService, AutoBackupAdmin)


class BackupPackagingAdmin(admin.ModelAdmin):
    list_display = ('name', 'id', 'download_button')
    ordering = ['id']

    def download_button(self, movie):
        url = reverse('admin-download-backup', args=[movie.id])
        button_html = f'<a class="button" href="{url}" target="_blank">Download</a>'
        return mark_safe(button_html)

    download_button.short_description = 'Download Backup'


admin.site.register(BackupPackaging, BackupPackagingAdmin)


class BackupUploadAdmin(admin.ModelAdmin):
    pass


admin.site.register(BackupUpload, BackupUploadAdmin)



# Generated by Django 4.2.5 on 2023-09-29 20:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_state_keeper', '0002_delete_backupupload_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='BackupUpload',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('zip_file', models.FileField(blank=True, upload_to='loaded_backup/')),
            ],
        ),
    ]



import os
import random
import string
import threading
from datetime import datetime
from threading import Thread
from time import sleep

import telegram
from django.db import models
from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver

from django_state_keeper.packaging_backup_model import BackupPackaging


class AutoBackupService(models.Model):
    THREADS_BY_ID = {}
    SHOULD_CONTINUE_BY_THREAD_NAME = {}
    name = models.CharField(max_length=100, blank=False, null=False)
    packaging = models.ForeignKey(BackupPackaging, blank=False, null=False, on_delete=models.CASCADE)
    period_minutes = models.IntegerField(default=24 * 60 * 60, blank=False, null=False)
    bot_token = models.CharField(max_length=100, blank=False, null=False)
    chat_id = models.CharField(max_length=50, blank=False, null=False)


@receiver(post_save, sender=AutoBackupService)
def create_backup_thread(sender, instance: AutoBackupService, **kwargs):
    stop_last_thread_of_backup_service(instance)
    new_thread_name = f'backup-sender-{instance.id}' + generate_random_string(10)
    AutoBackupService.SHOULD_CONTINUE_BY_THREAD_NAME[new_thread_name] = True
    new_thread = Thread(name=new_thread_name, target=backup_sender_thread, args=(instance,))
    AutoBackupService.THREADS_BY_ID[instance.id] = new_thread
    new_thread.start()


@receiver(pre_delete, sender=AutoBackupService)
def delete_backup_thread(sender, instance, **kwargs):
    stop_last_thread_of_backup_service(instance)


def stop_last_thread_of_backup_service(instance: AutoBackupService):
    if AutoBackupService.THREADS_BY_ID.__contains__(instance.id):
        stopping_thread: Thread = AutoBackupService.THREADS_BY_ID[instance.id]
        AutoBackupService.SHOULD_CONTINUE_BY_THREAD_NAME[stopping_thread.name] = False


def backup_sender_thread(instance: AutoBackupService):
    while AutoBackupService.SHOULD_CONTINUE_BY_THREAD_NAME[threading.current_thread().name]:
        send_backup_in_telegram(instance)
        sleep(60 * instance.period_minutes)


def send_backup_in_telegram(instance: AutoBackupService):
    bot = telegram.Bot(token=instance.bot_token)
    package_path = instance.packaging.create_a_backup_package()
    caption = f'{instance.name}\n{datetime.now().strftime("%Y/%m/%d %H:%M:%S")}'
    with open(package_path, 'rb') as file:
        bot.send_document(chat_id=instance.chat_id, document=file, caption=caption)
    os.remove(path=package_path)


def generate_random_string(length):
    characters = string.ascii_letters + string.digits + string.punctuation
    return ''.join(random.choice(characters) for _ in range(length))



from setuptools import setup, find_packages

setup(
    name='django-state-keeper',
    version='0.1',
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    description="A Django app that allows you to create backup of project and restore it later.",
    long_description=open('README.md').read(),
    url='https://github.com/MMDBadCoder/Django-State-Keeper',
    author='Mohammad Heydari',
    author_email='heidary13794@gmail.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=[
        'Django>=3.2',
        'python-telegram-bot>=20.5'
    ],
)



# Generated by Django 4.2.5 on 2023-09-29 20:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_state_keeper', '0001_initial'),
    ]

    operations = [
        migrations.DeleteModel(
            name='BackupUpload',
        ),
        migrations.AlterField(
            model_name='backuppackaging',
            name='paths_to_gather',
            field=models.TextField(default='db.sqlite3', max_length=1000),
        ),
    ]



import os
import shutil
import tempfile
import threading
from datetime import datetime, timedelta
from time import sleep

from django.conf import settings
from django.core.exceptions import BadRequest
from django.db import models


class BackupPackaging(models.Model):
    TIME_OF_ZIP_CREATION = {}
    name = models.CharField(max_length=100, blank=False, null=False)
    paths_to_gather = models.TextField(max_length=1000, blank=False, null=False, default='db.sqlite3')

    def create_a_backup_package(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # Copy pase mentioned files and directories to a temporary directory
            for relative_source_path in self.paths_to_gather.split('\n'):
                relative_source_path = relative_source_path.strip()
                if not relative_source_path:
                    continue
                absolute_source_path = os.path.join(settings.BASE_DIR, relative_source_path)
                if os.path.isfile(absolute_source_path):
                    absolute_target_path = os.path.join(temp_dir, relative_source_path)
                    os.makedirs(os.path.dirname(absolute_target_path), exist_ok=True)
                    shutil.copy(absolute_source_path, absolute_target_path)
                elif os.path.isdir(absolute_source_path):
                    absolute_target_directory = os.path.join(temp_dir, relative_source_path)
                    shutil.copytree(absolute_source_path, absolute_target_directory)
                else:
                    print("Invalid source path:", absolute_source_path)

            # Create a zip file containing all the files in the temporary directory
            zip_file_name = f'{self.name.replace(" ", "_")}-{datetime.now().strftime("%Y-%m-%d_%H-%M-%S")}'
            creating_zip_path = os.path.join(tempfile.tempdir, zip_file_name)
            creating_zip_path = shutil.make_archive(creating_zip_path, 'zip', temp_dir)

            if len(BackupPackaging.TIME_OF_ZIP_CREATION) == 0:
                threading.Thread(target=created_zips_cleaner).start()
            BackupPackaging.TIME_OF_ZIP_CREATION[creating_zip_path] = datetime.now()

            return creating_zip_path
        except Exception as e:
            raise BadRequest(e)


def created_zips_cleaner():
    hour_difference = timedelta(hours=1)
    while True:
        for created_zip_path, creation_time in BackupPackaging.TIME_OF_ZIP_CREATION.items():
            if datetime.now() - creation_time > hour_difference:
                os.remove(created_zip_path)
                del BackupPackaging.TIME_OF_ZIP_CREATION[created_zip_path]
        sleep(hour_difference.seconds)
